# """DataSpace URL Configuration

# The `urlpatterns` list routes URLs to views. For more information please see:
#     https://docs.djangoproject.com/en/4.0/topics/http/urls/
# Examples:
# Function views
#     1. Add an import:  from my_app import views
#     2. Add a URL to urlpatterns:  path('', views.home, name='home')
# Class-based views
#     1. Add an import:  from other_app.views import Home
#     2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
# Including another URLconf
#     1. Import the include() function: from django.urls import include, path
#     2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
# """
# from django.conf.urls.static import static
# from django.conf.urls import include
# from django.urls import path
# # from django.template.defaulttags import url
# from rest_framework.routers import DefaultRouter

# from DataSpace import settings
# from search.views import DatasetDocumentView, DatasetCompoundSearchBackendDocumentViewSet

# router = DefaultRouter()
# router.register(r'datasets',
#                     DatasetDocumentView,
#                     basename='datasetdocument')
# router.register(r'sdatasets',
#                     DatasetCompoundSearchBackendDocumentViewSet,
#                     basename='sdatasetdocument')

# urlpatterns = [
#                   path('', include(router.urls)),
#               ] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
